/* =================================================================================
 *  parentetica.c
 * =================================================================================
 * Contiene l'implementazione delle seguenti funzioni e procedure:
 * - costruzione di una lista di caratteri
 * - stampa del contenuto di una lista di caratteri
 * - stampa indentata di un albero binario
 * - verifica se una lista contiene una rappresentazione parentetica legale di un 
 *   albero binario, e in tal caso costruisce l'albero
 *
 *  Authors:     Irene Finocchi  (finocchi@di.uniroma1.it)
 *  Last change: 16 Aprile 2005
 *  Changed by:  Irene Finocchi
*/

#include "parentetica.h"

/* Pre: la lista che si costruisce e' NON vuota */
/* Post: costruisce una lista di caratteri */
ListPtr crealista() {
	ListPtr L;
	char c;

	scanf("%c",&c);
	if (c=='\n') return NULL;
	L = malloc(sizeof(struct nodolista));	
	if (!L) {
		printf("Allocazione di memoria fallita nella creazione della lista");
		return NULL;
	}
	L->elem = c;
	L->next = crealista();

	return(L);
}

/* Post: stampa il contenuto della lista */
void stampaLista(ListPtr L) {
	if (L!=NULL) {
		printf("%c ",L->elem);
		stampaLista(L->next);
	}
	return;
}

/* OSSERVAZIONE: nel main chiamata con h=0 */
/* Post: realizza una stampa indentata di un albero binario */
void stampaIndentata (TreePtr T, int h) {
	int i;
	if (T!=NULL) {
		stampaIndentata(T->rPtr,h+1);
		for (i=1;i<=h;i++)
			printf("\t");
		printf("%c \n", T->elem);
		stampaIndentata(T->lPtr,h+1);
	}
	
}

/* Post: verifica se il contenuto della lista e' una rappresentazione parentetica
   legale di un albero binario: se si, costruisce l'albero; se no, restituisce NULL
   e stampa il prefisso legale della stringa */
TreePtr ListToTree(ListPtr L) {
	TreePtr T;
	ListPtr L1=L;
	if (ListToTreeAux(&L,&T)==1 && L==NULL) return T;
	printf("\nLa rappresentazione parentetica non e' legale.\n");
	if (L1==L) printf("Il prefisso legale e' vuoto.");
	else {
		printf("Il prefisso legale e': ");
		while (L1!=L) {
			printf("%c ",L1->elem);
			L1=L1->next;
		}
	}
	return NULL;
}

/* Metodo ausiliario ricorsivo per la costruzione di un albero dalla rappresentazione parentetica */
int ListToTreeAux(ListPtr *L, TreePtr *T) {

	/* Rappresentazione non legale */
	*T = NULL;
	if (*L==NULL || (*L)->elem!='(' || (*L)->next==NULL) return 0;

	*L = (*L)->next;
	if (*L==NULL) return 0;

	/* Albero vuoto */
	if ((*L)->elem==')') {
		*L = (*L)->next;
		return 1;
	}
	
	/* Albero non vuoto: alloca spazio per un nuovo nodo */
	*T = malloc(sizeof(struct nodoalbero));
	if (*T==NULL) {
		printf("Allocazione di memoria fallita in ListToTreeAux().\n");
		return 0;
	}
	(*T)->elem=(*L)->elem;
	*L = (*L)->next;
	if (ListToTreeAux(L,&((*T)->lPtr))==0) return 0;
	if (ListToTreeAux(L,&((*T)->rPtr))==0) return 0;
	
	/* Verifica la presenza di una parentesi chiusa */
	if (*L==NULL || (*L)->elem!=')') return 0;
	
	*L = (*L)->next;
	return 1;
}









